module PostDB
	module CLI
    # The helper CLI module
    #
    # This module contains helper functionality for the CLI.
    #
		module Helper
			# Print a newline character
			#
			# Example:
			#   >> new_line
			#   => "\n"
			#
			def new_line
				puts ""
			end

			# Get a prompt object
			#
			# Example:
			#   >> prompt
			#   => #<TTY::Prompt:0x00000000000000>
			#
			def prompt
				@prompt ||= TTY::Prompt.new
			end

			# Exit with an error
			#
			# Arguments:
			#   messages: (Splat)
			#
			# Example:
			#   >> exit_with_error "Oops!"
			#   => nil
			#
			def exit_with_error(*messages)
				messages.each do |message|
					("\n" == message) ? print(message) : prompt.error(message)
				end

				exit 1
			end

			# Exit with a warning
			#
			# Arguments:
			#   messages: (Splat)
			#
			# Example:
			#   >> exit_with_warning "Oops!"
			#   => nil
			#
			def exit_with_warning(*messages)
				messages.each do |message|
					("\n" == message) ? print(message) : prompt.warn(message)
				end

				exit 1
			end

			# Confirm an action
			#
			# Arguments:
			#   message: (String)
			#   aborted_message: (String)
			#
			# Example:
			#   >> confirm_action! "Do something?"
			#   => nil
			#
			def confirm_action!(message, aborted_message = nil)
				# Prompt the user to confirm the action
				if prompt.no?(message)
					# Exit with a warning
					exit_with_warning(aborted_message)
				end
			end

			# Format an error
			#
			# Arguments:
			#   error: (Exception)
			#
			# Example:
			#   >> format_error(error)
			#   => ["\n", "Error:", "  Error Message", "Backtrace:", "  from (irb):1"]
			#
			def format_error(error)
				errors = Array.new
				errors << "\n"
				errors << "Error:"
				errors << "  " + error.message
				errors << "\n"
				errors << "Backtrace:"
				errors += error.backtrace.map { |trace| "  " + trace }
				errors
			end
		end
	end
end
