module PostDB
  class Domain < ActiveRecord::Base
    self.table_name = 'virtual_domains'

    serialize :dkim, OpenSSL::PKey::RSA

    has_many :users,   dependent: :destroy
    has_many :aliases, dependent: :destroy

    validates :name, presence: true, uniqueness: true
    validates :dkim, presence: true, uniqueness: true

    before_validation(on: :create) do |domain|
      domain.regenerate_dkim
    end

    after_save do
      PostDB::DKIM.generate_configuration
    end

    after_destroy do
      PostDB::DKIM.generate_configuration
    end

    # Get all forwarding aliases
    # This excludes aliases where source == destination
    #
    # Example:
    #   >> domain.forwarding_aliases
    #   => []
    #
    def forwarding_aliases
      # self.aliases.select { |a| a.source != a.destination }
      self.aliases.where('source != destination')
    end

    # Get the path to the DKIM key file
    #
    # Example:
    #   >> domain.dkim_path
    #   => "..."
    #
    def dkim_path
      keys_directory = PostDB::DKIM.keys_directory

      File.join(keys_directory, "#{self.name}.private")
    end

    # Generate a new DKIM key
    #
    # Arguments:
    #   size: (Integer) (Default: 2048)
    #
    # Example:
    #   >> domain.regenerate_dkim
    #   => #<OpenSSL::PKey::RSA:0x00000000000000>
    #
    def regenerate_dkim(size = 2048)
      self.dkim = OpenSSL::PKey::RSA.new(size)
    end
  end
end
